#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: reserve_ip_subpool_info
short_description: Information module for Reserve Ip Subpool
description:
- Get all Reserve Ip Subpool.
- API to get the ip subpool info.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - SiteId query parameter. Site id to get the reserve ip associated with the site.
    type: str
  offset:
    description:
    - Offset query parameter. Offset/starting row.
    type: int
  limit:
    description:
    - Limit query parameter. No of Global Pools to be retrieved.
    type: int
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings GetReserveIPSubpool
  description: Complete reference of the GetReserveIPSubpool API.
  link: https://developer.cisco.com/docs/dna-center/#!get-reserve-ip-subpool
notes:
  - SDK Method used are
    network_settings.NetworkSettings.get_reserve_ip_subpool,

  - Paths used are
    get /dna/intent/api/v1/reserve-ip-subpool,

"""

EXAMPLES = r"""
- name: Get all Reserve Ip Subpool
  cisco.dnac.reserve_ip_subpool_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    offset: 0
    limit: 0
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "groupName": "string",
          "ipPools": [
            {
              "ipPoolName": "string",
              "dhcpServerIps": [
                {}
              ],
              "gateways": [
                "string"
              ],
              "createTime": 0,
              "lastUpdateTime": 0,
              "totalIpAddressCount": 0,
              "usedIpAddressCount": 0,
              "parentUuid": "string",
              "owner": "string",
              "shared": true,
              "overlapping": true,
              "configureExternalDhcp": true,
              "usedPercentage": "string",
              "clientOptions": {},
              "groupUuid": "string",
              "dnsServerIps": [
                {}
              ],
              "context": [
                {
                  "owner": "string",
                  "contextKey": "string",
                  "contextValue": "string"
                }
              ],
              "ipv6": true,
              "id": "string",
              "ipPoolCidr": "string"
            }
          ],
          "siteId": "string",
          "siteHierarchy": "string",
          "type": "string",
          "groupOwner": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_ssids_firewall_l7_firewall_rules
short_description: Resource module for networks _wireless _ssids _firewall l7firewallrules
description:
- Manage operation update of the resource networks _wireless _ssids _firewall l7firewallrules.
- Update the L7 firewall rules of an SSID on an MR network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  rules:
    description: An array of L7 firewall rules for this SSID. Rules will get applied
      in the same order user has specified in request. Empty array will clear the L7
      firewall rule configuration.
    elements: dict
    suboptions:
      policy:
        description: '''Deny'' traffic specified by this rule.'
        type: str
      type:
        description: Type of the L7 firewall rule. One of 'application', 'applicationCategory',
          'host', 'port', 'ipRange'.
        type: str
      value:
        description: The value of what needs to get blocked. Format of the value varies
          depending on type of the firewall rule selected.
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless updateNetworkWirelessSsidFirewallL7FirewallRules
  description: Complete reference of the updateNetworkWirelessSsidFirewallL7FirewallRules API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ssid-firewall-l7-firewall-rules
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_ssid_firewall_l7_firewall_rules,

  - Paths used are
    put /networks/{networkId}/wireless/ssids/{number}/firewall/l7FirewallRules,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_wireless_ssids_firewall_l7_firewall_rules:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    networkId: string
    number: string
    rules:
    - policy: deny
      type: host
      value: google.com
    - policy: deny
      type: port
      value: '23'
    - policy: deny
      type: ipRange
      value: 10.11.12.00/24
    - policy: deny
      type: ipRange
      value: 10.11.12.00/24:5555

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: slaac
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import (
    _need_netaddr,
    hwaddr,
    ipaddr,
)


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: slaac
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: This filter returns the SLAAC address within a network for a given HW/MAC address.
    description:
    - This filter returns the SLAAC address within a network for a given HW/MAC address.
    - The filter slaac() generates an IPv6 address for a given network and a MAC Address in Stateless Configuration.
    options:
        value:
            description: The network address or range to test against.
            type: str
            required: True
        query:
            description: nth host
            type: str
    notes:
"""

EXAMPLES = r"""
#### examples
- name: The filter slaac() generates an IPv6 address for a given network and a MAC Address in Stateless Configuration.
  debug:
    msg: "{{ 'fdcf:1894:23b5:d38c:0000:0000:0000:0000' | slaac('c2:31:b3:83:bf:2b') }}"

# TASK [The filter slaac() generates an IPv6 address for a given network and a MAC Address in Stateless Configuration.] ***
# task path: /Users/amhatre/ansible-collections/playbooks/test_slaac.yaml:7
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "fdcf:1894:23b5:d38c:c031:b3ff:fe83:bf2b"
# }
"""

RETURN = """
  data:
    type: str
    description:
      - Returns the SLAAC address within a network for a given HW/MAC address.

"""


@pass_environment
def _slaac(*args, **kwargs):
    """This filter returns whether an address or a network passed as argument is in a network."""
    keys = ["value", "query"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="slaac")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return slaac(**updated_data)


def slaac(value, query=""):
    """Get the SLAAC address within given network"""
    try:
        vtype = ipaddr(value, "type")
        if vtype == "address":
            v = ipaddr(value, "cidr")
        elif vtype == "network":
            v = ipaddr(value, "subnet")

        if ipaddr(value, "version") != 6:
            return False

        value = netaddr.IPNetwork(v)
    except Exception:
        return False

    if not query:
        return False

    try:
        mac = hwaddr(query, alias="slaac")

        eui = netaddr.EUI(mac)
    except Exception:
        return False

    return str(eui.ipv6(value.network))


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "slaac": _slaac,
    }

    def filters(self):
        """ipaddr filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_policy_objects_info
short_description: Information module for organizations _policyobjects
description:
- Get all organizations _policyobjects.
- Get organizations _policyobjects by id.
- Lists Policy Objects belonging to the organization.
- Shows details of a Policy Object.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
    - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
    - PerPage query parameter. The number of entries per page returned. Acceptable range is 10 - 5000. Default is 5000.
    type: int
  startingAfter:
    description:
    - >
      StartingAfter query parameter. A token used by the server to indicate the start of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  endingBefore:
    description:
    - >
      EndingBefore query parameter. A token used by the server to indicate the end of the page. Often this is a
      timestamp or an ID but it is not limited to those. This parameter should not be defined by client
      applications. The link for the first, last, prev, or next page in the HTTP Link header should define it.
    type: str
  policyObjectId:
    description:
    - PolicyObjectId path parameter. Policy object ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations getOrganizationPolicyObject
  description: Complete reference of the getOrganizationPolicyObject API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-organization-policy-object
- name: Cisco Meraki documentation for organizations getOrganizationPolicyObjects
  description: Complete reference of the getOrganizationPolicyObjects API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-organization-policy-objects
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_policy_object,
    organizations.Organizations.get_organization_policy_objects,

  - Paths used are
    get /organizations/{organizationId}/policyObjects,
    get /organizations/{organizationId}/policyObjects/{policyObjectId},
"""

EXAMPLES = r"""
- name: Get all organizations _policyobjects
  cisco.meraki.organizations_policy_objects_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result

- name: Get organizations _policyobjects by id
  cisco.meraki.organizations_policy_objects_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    organizationId: string
    policyObjectId: string
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {}
    ]
"""



# -*- coding: utf-8 -*-

# (c) 2022 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

try:
    import botocore
except ImportError:
    pass  # will be captured by imported HAS_BOTO3

from ansible.module_utils.basic import to_native

from ansible_collections.amazon.aws.plugins.module_utils.botocore import _aws_connection_info
from ansible_collections.amazon.aws.plugins.module_utils.botocore import _aws_region
from ansible_collections.amazon.aws.plugins.module_utils.botocore import _boto3_conn
from ansible_collections.amazon.aws.plugins.module_utils.exceptions import AnsibleBotocoreError


def boto3_conn(plugin, conn_type=None, resource=None, region=None, endpoint=None, **params):
    """
    Builds a boto3 resource/client connection cleanly wrapping the most common failures.
    Handles:
        ValueError,
        botocore.exceptions.ProfileNotFound, botocore.exceptions.PartialCredentialsError,
        botocore.exceptions.NoCredentialsError, botocore.exceptions.ConfigParseError,
        botocore.exceptions.NoRegionError
    """

    try:
        return _boto3_conn(conn_type=conn_type, resource=resource, region=region, endpoint=endpoint, **params)
    except ValueError as e:
        plugin.fail_aws(f"Couldn't connect to AWS: {to_native(e)}")
    except (
        botocore.exceptions.ProfileNotFound,
        botocore.exceptions.PartialCredentialsError,
        botocore.exceptions.NoCredentialsError,
        botocore.exceptions.ConfigParseError,
    ) as e:
        plugin.fail_aws(to_native(e))
    except botocore.exceptions.NoRegionError:
        # ansible_name is added in 2.14
        if hasattr(plugin, "ansible_name"):
            plugin.fail_aws(
                f"The {plugin.ansible_name} plugin requires a region and none was found in configuration, "
                "environment variables or module parameters"
            )
        plugin.fail_aws(
            "A region is required and none was found in configuration, environment variables or module parameters"
        )


def get_aws_connection_info(plugin):
    try:
        return _aws_connection_info(plugin.get_options())
    except AnsibleBotocoreError as e:
        plugin.fail_aws(to_native(e))


def get_aws_region(plugin):
    try:
        return _aws_region(plugin.get_options())
    except AnsibleBotocoreError as e:
        plugin.fail_aws(to_native(e))



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lightsail_snapshot
version_added: "6.0.0"
short_description: Creates snapshots of AWS Lightsail instances
description:
    - Creates snapshots of AWS Lightsail instances.
author:
    - "Nuno Saavedra (@Nfsaavedra)"
options:
  state:
    description:
      - Indicate desired state of the target.
    default: present
    choices: ['present', 'absent']
    type: str
  snapshot_name:
    description: Name of the new instance snapshot.
    required: true
    type: str
  instance_name:
    description:
      - Name of the instance to create the snapshot.
      - Required when I(state=present).
    type: str
  wait:
    description:
      - Wait for the instance snapshot to be created before returning.
    type: bool
    default: true
  wait_timeout:
    description:
      - How long before I(wait) gives up, in seconds.
    default: 300
    type: int

extends_documentation_fragment:
- amazon.aws.common.modules
- amazon.aws.region.modules
- amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create AWS Lightsail snapshot
  lightsail_snapshot:
    region: us-east-1
    snapshot_name: "my_instance_snapshot"
    instance_name: "my_instance"

- name: Delete AWS Lightsail snapshot
  lightsail_snapshot:
    region: us-east-1
    snapshot_name: "my_instance_snapshot"
    state: absent
"""

RETURN = r"""
changed:
  description: if a snapshot has been modified/created
  returned: always
  type: bool
  sample:
    changed: true
snapshot:
  description: instance snapshot data
  type: dict
  returned: always
  sample:
    arn: "arn:aws:lightsail:us-east-1:070807442430:InstanceSnapshot/54b0f785-7132-443d-9e32-95a6825636a4"
    created_at: "2023-02-23T18:46:11.183000+00:00"
    from_attached_disks: []
    from_blueprint_id: "amazon_linux_2"
    from_bundle_id: "nano_2_0"
    from_instance_arn: "arn:aws:lightsail:us-east-1:070807442430:Instance/5ca1e7ca-a994-4e19-bb82-deb9d79e9ca3"
    from_instance_name: "my_instance"
    is_from_auto_snapshot: false
    location:
      availability_zone: "all"
      region_name: "us-east-1"
    name: "my_instance_snapshot"
    resource_type: "InstanceSnapshot"
    size_in_gb: 20
    state: "available"
    support_code: "351201681302/ami-06b48e5589f1e248b"
    tags: []
"""

import time

try:
    import botocore
except ImportError:
    # will be caught by AnsibleAWSModule
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.core import is_boto3_error_code

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def find_instance_snapshot_info(module, client, instance_snapshot_name, fail_if_not_found=False):
    try:
        res = client.get_instance_snapshot(instanceSnapshotName=instance_snapshot_name)
    except is_boto3_error_code("NotFoundException") as e:
        if fail_if_not_found:
            module.fail_json_aws(e)
        return None
    except botocore.exceptions.ClientError as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)
    return res["instanceSnapshot"]


def wait_for_instance_snapshot(module, client, instance_snapshot_name):
    wait_timeout = module.params.get("wait_timeout")
    wait_max = time.time() + wait_timeout
    snapshot = find_instance_snapshot_info(module, client, instance_snapshot_name)

    while wait_max > time.time():
        snapshot = find_instance_snapshot_info(module, client, instance_snapshot_name)
        current_state = snapshot["state"]
        if current_state != "pending":
            break
        time.sleep(5)
    else:
        module.fail_json(msg=f'Timed out waiting for instance snapshot "{instance_snapshot_name}" to be created.')

    return snapshot


def create_snapshot(module, client):
    snapshot = find_instance_snapshot_info(module, client, module.params.get("snapshot_name"))
    new_instance = snapshot is None

    if module.check_mode or not new_instance:
        snapshot = snapshot if snapshot is not None else {}
        module.exit_json(
            changed=new_instance,
            instance_snapshot=camel_dict_to_snake_dict(snapshot),
        )

    try:
        snapshot = client.create_instance_snapshot(
            instanceSnapshotName=module.params.get("snapshot_name"),
            instanceName=module.params.get("instance_name"),
        )
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    if module.params.get("wait"):
        snapshot = wait_for_instance_snapshot(module, client, module.params.get("snapshot_name"))

    module.exit_json(
        changed=new_instance,
        instance_snapshot=camel_dict_to_snake_dict(snapshot),
    )


def delete_snapshot(module, client):
    snapshot = find_instance_snapshot_info(module, client, module.params.get("snapshot_name"))
    if module.check_mode or snapshot is None:
        changed = not (snapshot is None)
        instance = snapshot if changed else {}
        module.exit_json(changed=changed, instance=instance)

    try:
        client.delete_instance_snapshot(instanceSnapshotName=module.params.get("snapshot_name"))
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    module.exit_json(changed=True, instance=camel_dict_to_snake_dict(snapshot))


def main():
    argument_spec = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        snapshot_name=dict(type="str", required=True),
        instance_name=dict(type="str"),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(default=300, type="int"),
    )
    required_if = [
        ["state", "present", ("instance_name",)],
    ]

    module = AnsibleAWSModule(argument_spec=argument_spec, required_if=required_if, supports_check_mode=True)
    client = module.client("lightsail")

    state = module.params.get("state")

    if state == "present":
        create_snapshot(module, client)
    elif state == "absent":
        delete_snapshot(module, client)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_webhooks_payload_templates
short_description: Resource module for networks _webhooks _payloadtemplates
description:
- Manage operations create, update and delete of the resource networks _webhooks _payloadtemplates.
- Create a webhook payload template for a network.
- >
   Destroy a webhook payload template for a network. Does not work for included templates 'wpt_00001', 'wpt_00002',
   'wpt_00003', 'wpt_00004', 'wpt_00005' or 'wpt_00006' .
- Update a webhook payload template for a network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  body:
    description: The liquid template used for the body of the webhook message. Either
      `body` or `bodyFile` must be specified.
    type: str
  bodyFile:
    description: A file containing liquid template used for the body of the webhook
      message. Either `body` or `bodyFile` must be specified.
    type: str
  headers:
    description: The liquid template used with the webhook headers.
    elements: dict
    suboptions:
      name:
        description: The name of the header template.
        type: str
      template:
        description: The liquid template for the headers.
        type: str
    type: list
  headersFile:
    description: A file containing the liquid template used with the webhook headers.
    type: str
  name:
    description: The name of the new template.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  payloadTemplateId:
    description: PayloadTemplateId path parameter. Payload template ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks createNetworkWebhooksPayloadTemplate
  description: Complete reference of the createNetworkWebhooksPayloadTemplate API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-webhooks-payload-template
- name: Cisco Meraki documentation for networks deleteNetworkWebhooksPayloadTemplate
  description: Complete reference of the deleteNetworkWebhooksPayloadTemplate API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-webhooks-payload-template
- name: Cisco Meraki documentation for networks updateNetworkWebhooksPayloadTemplate
  description: Complete reference of the updateNetworkWebhooksPayloadTemplate API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-webhooks-payload-template
notes:
  - SDK Method used are
    networks.Networks.create_network_webhooks_payload_template,
    networks.Networks.delete_network_webhooks_payload_template,
    networks.Networks.update_network_webhooks_payload_template,

  - Paths used are
    post /networks/{networkId}/webhooks/payloadTemplates,
    delete /networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId},
    put /networks/{networkId}/webhooks/payloadTemplates/{payloadTemplateId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    body: '{"event_type":"{{alertTypeId}}","client_payload":{"text":"{{alertData}}"}}'
    bodyFile: Qm9keSBGaWxl
    headers:
    - name: Authorization
      template: Bearer {{sharedSecret}}
    headersFile: SGVhZGVycyBGaWxl
    name: Custom Template
    networkId: string

- name: Update by id
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    body: '{"event_type":"{{alertTypeId}}","client_payload":{"text":"{{alertData}}"}}'
    bodyFile: Qm9keSBGaWxl
    headers:
    - name: Authorization
      template: Bearer {{sharedSecret}}
    headersFile: SGVhZGVycyBGaWxl
    name: Custom Template
    networkId: string
    payloadTemplateId: string

- name: Delete by id
  cisco.meraki.networks_webhooks_payload_templates:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string
    payloadTemplateId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "payloadTemplateId": "string",
      "type": "string",
      "name": "string",
      "headers": [
        {
          "name": "string",
          "template": "string"
        }
      ],
      "body": "string",
      "sharing": {
        "byNetwork": {
          "adminsCanModify": true
        }
      }
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the nxos_logging_global module
"""


class Logging_globalArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_logging_global module"""

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "dict",
            "options": {
                "console": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                    },
                },
                "event": {
                    "type": "dict",
                    "options": {
                        "link_status": {
                            "type": "dict",
                            "options": {
                                "enable": {"type": "bool"},
                                "default": {"type": "bool"},
                            },
                        },
                        "trunk_status": {
                            "type": "dict",
                            "options": {
                                "enable": {"type": "bool"},
                                "default": {"type": "bool"},
                            },
                        },
                    },
                },
                "history": {
                    "type": "dict",
                    "options": {
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                        "size": {"type": "int"},
                    },
                },
                "ip": {
                    "type": "dict",
                    "options": {
                        "access_list": {
                            "type": "dict",
                            "options": {
                                "cache": {
                                    "type": "dict",
                                    "options": {
                                        "entries": {"type": "int"},
                                        "interval": {"type": "int"},
                                        "threshold": {"type": "int"},
                                    },
                                },
                                "detailed": {"type": "bool"},
                                "include": {
                                    "type": "dict",
                                    "options": {"sgt": {"type": "bool"}},
                                },
                            },
                        },
                    },
                },
                "facilities": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "facility": {"type": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                    },
                },
                "logfile": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "name": {"type": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                        "persistent_threshold": {"type": "int"},
                        "size": {"type": "int"},
                    },
                },
                "module": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                    },
                },
                "monitor": {
                    "type": "dict",
                    "options": {
                        "state": {
                            "type": "str",
                            "choices": ["enabled", "disabled"],
                        },
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                    },
                },
                "origin_id": {
                    "type": "dict",
                    "options": {
                        "hostname": {"type": "bool"},
                        "ip": {"type": "str"},
                        "string": {"type": "str"},
                    },
                },
                "rate_limit": {
                    "type": "str",
                    "choices": ["enabled", "disabled"],
                },
                "rfc_strict": {"type": "bool"},
                "hosts": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "host": {"type": "str"},
                        "severity": {
                            "type": "str",
                            "choices": [
                                "emergency",
                                "alert",
                                "critical",
                                "error",
                                "warning",
                                "notification",
                                "informational",
                                "debugging",
                            ],
                        },
                        "facility": {"type": "str"},
                        "port": {"type": "int"},
                        "secure": {
                            "type": "dict",
                            "options": {
                                "trustpoint": {
                                    "type": "dict",
                                    "options": {"client_identity": {"type": "str"}},
                                },
                            },
                        },
                        "use_vrf": {"type": "str"},
                    },
                },
                "source_interface": {"type": "str"},
                "timestamp": {
                    "type": "str",
                    "choices": ["microseconds", "milliseconds", "seconds"],
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The facts class for eos
this file validates each subset of facts and selectively
calls the appropriate facts gathering function
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.facts.facts import (
    FactsBase,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.acl_interfaces.acl_interfaces import (
    Acl_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.acls.acls import (
    AclsFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.bgp_address_family.bgp_address_family import (
    Bgp_afFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.bgp_global.bgp_global import (
    Bgp_globalFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.hostname.hostname import (
    HostnameFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.interfaces.interfaces import (
    InterfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.l2_interfaces.l2_interfaces import (
    L2_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.l3_interfaces.l3_interfaces import (
    L3_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.lacp.lacp import (
    LacpFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.lacp_interfaces.lacp_interfaces import (
    Lacp_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.lag_interfaces.lag_interfaces import (
    Lag_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.legacy.base import (
    Config,
    Default,
    Hardware,
    Interfaces,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.lldp_global.lldp_global import (
    Lldp_globalFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.lldp_interfaces.lldp_interfaces import (
    Lldp_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.logging_global.logging_global import (
    Logging_globalFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.ntp_global.ntp_global import (
    Ntp_globalFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.ospf_interfaces.ospf_interfaces import (
    Ospf_interfacesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.ospfv2.ospfv2 import (
    Ospfv2Facts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.ospfv3.ospfv3 import (
    Ospfv3Facts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.prefix_lists.prefix_lists import (
    Prefix_listsFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.route_maps.route_maps import (
    Route_mapsFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.snmp_server.snmp_server import (
    Snmp_serverFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.static_routes.static_routes import (
    Static_routesFacts,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.vlans.vlans import (
    VlansFacts,
)


FACT_LEGACY_SUBSETS = dict(
    default=Default,
    hardware=Hardware,
    interfaces=Interfaces,
    config=Config,
)
FACT_RESOURCE_SUBSETS = dict(
    interfaces=InterfacesFacts,
    l2_interfaces=L2_interfacesFacts,
    l3_interfaces=L3_interfacesFacts,
    lacp=LacpFacts,
    lacp_interfaces=Lacp_interfacesFacts,
    lag_interfaces=Lag_interfacesFacts,
    lldp_global=Lldp_globalFacts,
    lldp_interfaces=Lldp_interfacesFacts,
    vlans=VlansFacts,
    acl_interfaces=Acl_interfacesFacts,
    acls=AclsFacts,
    static_routes=Static_routesFacts,
    ospfv2=Ospfv2Facts,
    ospfv3=Ospfv3Facts,
    ospf_interfaces=Ospf_interfacesFacts,
    bgp_address_family=Bgp_afFacts,
    bgp_global=Bgp_globalFacts,
    route_maps=Route_mapsFacts,
    prefix_lists=Prefix_listsFacts,
    logging_global=Logging_globalFacts,
    ntp_global=Ntp_globalFacts,
    snmp_server=Snmp_serverFacts,
    hostname=HostnameFacts,
)


class Facts(FactsBase):
    """The fact class for eos"""

    VALID_LEGACY_GATHER_SUBSETS = frozenset(FACT_LEGACY_SUBSETS.keys())
    VALID_RESOURCE_SUBSETS = frozenset(FACT_RESOURCE_SUBSETS.keys())

    def get_facts(
        self,
        legacy_facts_type=None,
        resource_facts_type=None,
        data=None,
    ):
        """Collect the facts for eos
        :param legacy_facts_type: List of legacy facts types
        :param resource_facts_type: List of resource fact types
        :param data: previously collected conf
        :rtype: dict
        :return: the facts gathered
        """
        if self.VALID_RESOURCE_SUBSETS:
            self.get_network_resources_facts(
                FACT_RESOURCE_SUBSETS,
                resource_facts_type,
                data,
            )

        if self.VALID_LEGACY_GATHER_SUBSETS:
            self.get_network_legacy_facts(
                FACT_LEGACY_SUBSETS,
                legacy_facts_type,
            )
        return self.ansible_facts, self._warnings



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# Copyright: (c) 2021, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_filter_entry
short_description: Manage filter entries in schema templates
description:
- Manage filter entries in schema templates on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
- Anvitha Jain (@anvitha-jain)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  filter:
    description:
    - The name of the filter to manage.
    - There should be no space in the filter name. APIC will throw an error if a space is provided in the filter name.
    - See the C(filter_display_name) attribute if you want the display name of the filter to contain a space.
    type: str
    required: true
  filter_description:
    description:
    - The description of this filter is supported on versions of MSO that are 3.3 or greater.
    type: str
    default: ''
  filter_display_name:
    description:
    - The name as displayed on the MSO web interface.
    type: str
  entry:
    description:
    - The filter entry name to manage.
    type: str
    aliases: [ name ]
  display_name:
    description:
    - The name as displayed on the MSO web interface.
    type: str
    aliases: [ entry_display_name ]
  filter_entry_description:
    description:
    - The description of this filter entry.
    type: str
    aliases: [ entry_description, description ]
    default: ''
  ethertype:
    description:
    - The ethernet type to use for this filter entry.
    type: str
    choices: [ arp, fcoe, ip, ipv4, ipv6, mac-security, mpls-unicast, trill, unspecified ]
  ip_protocol:
    description:
    - The IP protocol to use for this filter entry.
    type: str
    choices: [ eigrp, egp, icmp, icmpv6, igmp, igp, l2tp, ospfigp, pim, tcp, udp, unspecified ]
  tcp_session_rules:
    description:
    - A list of TCP session rules.
    type: list
    elements: str
    choices: [ acknowledgement, established, finish, synchronize, reset, unspecified ]
  source_from:
    description:
    - The source port range from.
    type: str
  source_to:
    description:
    - The source port range to.
    type: str
  destination_from:
    description:
    - The destination port range from.
    type: str
  destination_to:
    description:
    - The destination port range to.
    type: str
  arp_flag:
    description:
    - The ARP flag to use for this filter entry.
    type: str
    choices: [ reply, request, unspecified ]
  stateful:
    description:
    - Whether this filter entry is stateful.
    type: bool
  fragments_only:
    description:
    - Whether this filter entry only matches fragments.
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_contract_filter
notes:
- Due to restrictions of the MSO REST API this module creates filters when needed, and removes them when the last entry has been removed.
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new filter entry
  cisco.mso.mso_schema_template_filter_entry:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    filter: Filter 1
    state: present
  delegate_to: localhost

- name: Remove a filter entry
  cisco.mso.mso_schema_template_filter_entry:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    filter: Filter 1
    state: absent
  delegate_to: localhost

- name: Query a specific filter entry
  cisco.mso.mso_schema_template_filter_entry:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    filter: Filter 1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all filter entries
  cisco.mso.mso_schema_template_filter_entry:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        filter=dict(type="str", required=True),
        filter_description=dict(type="str", default=""),
        filter_display_name=dict(type="str"),
        entry=dict(type="str", aliases=["name"]),  # This parameter is not required for querying all objects
        filter_entry_description=dict(type="str", default="", aliases=["entry_description", "description"]),
        display_name=dict(type="str", aliases=["entry_display_name"]),
        ethertype=dict(type="str", choices=["arp", "fcoe", "ip", "ipv4", "ipv6", "mac-security", "mpls-unicast", "trill", "unspecified"]),
        ip_protocol=dict(type="str", choices=["eigrp", "egp", "icmp", "icmpv6", "igmp", "igp", "l2tp", "ospfigp", "pim", "tcp", "udp", "unspecified"]),
        tcp_session_rules=dict(type="list", elements="str", choices=["acknowledgement", "established", "finish", "synchronize", "reset", "unspecified"]),
        source_from=dict(type="str"),
        source_to=dict(type="str"),
        destination_from=dict(type="str"),
        destination_to=dict(type="str"),
        arp_flag=dict(type="str", choices=["reply", "request", "unspecified"]),
        stateful=dict(type="bool"),
        fragments_only=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["entry"]],
            ["state", "present", ["entry"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    filter_name = module.params.get("filter")
    filter_display_name = module.params.get("filter_display_name")
    filter_description = module.params.get("filter_description")
    entry = module.params.get("entry")
    display_name = module.params.get("display_name")
    filter_entry_description = module.params.get("filter_entry_description")
    ethertype = module.params.get("ethertype")
    ip_protocol = module.params.get("ip_protocol")
    tcp_session_rules = module.params.get("tcp_session_rules")
    source_from = module.params.get("source_from")
    source_to = module.params.get("source_to")
    destination_from = module.params.get("destination_from")
    destination_to = module.params.get("destination_to")
    arp_flag = module.params.get("arp_flag")
    if arp_flag == "request":
        arp_flag = "req"
    stateful = module.params.get("stateful")
    fragments_only = module.params.get("fragments_only")
    state = module.params.get("state")

    mso = MSOModule(module)

    # Get schema
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    templates = [t.get("name") for t in schema_obj.get("templates")]
    if template not in templates:
        mso.fail_json(
            msg="Provided template '{template}' does not exist. Existing templates: {templates}".format(template=template, templates=", ".join(templates))
        )
    template_idx = templates.index(template)

    # Get filters
    mso.existing = {}
    filter_idx = None
    entry_idx = None
    filters = [f.get("name") for f in schema_obj.get("templates")[template_idx]["filters"]]
    if filter_name in filters:
        filter_idx = filters.index(filter_name)

        entries = [f.get("name") for f in schema_obj.get("templates")[template_idx]["filters"][filter_idx]["entries"]]
        if entry in entries:
            entry_idx = entries.index(entry)
            mso.existing = schema_obj.get("templates")[template_idx]["filters"][filter_idx]["entries"][entry_idx]

    if state == "query":
        if entry is None:
            if filter_idx is None:
                mso.fail_json(msg="Filter '{filter}' not found".format(filter=filter_name))
            mso.existing = schema_obj.get("templates")[template_idx]["filters"][filter_idx]["entries"]
        elif not mso.existing:
            mso.fail_json(msg="Entry '{entry}' not found".format(entry=entry))
        mso.exit_json()

    filters_path = "/templates/{0}/filters".format(template)
    filter_path = "/templates/{0}/filters/{1}".format(template, filter_name)
    entries_path = "/templates/{0}/filters/{1}/entries".format(template, filter_name)
    entry_path = "/templates/{0}/filters/{1}/entries/{2}".format(template, filter_name, entry)
    ops = []

    mso.previous = mso.existing
    if state == "absent":
        mso.proposed = mso.sent = {}

        if filter_idx is None:
            # There was no filter to begin with
            pass
        elif entry_idx is None:
            # There was no entry to begin with
            pass
        elif len(entries) == 1:
            # There is only one entry, remove filter
            mso.existing = {}
            ops.append(dict(op="remove", path=filter_path))

        else:
            mso.existing = {}
            ops.append(dict(op="remove", path=entry_path))

    elif state == "present":
        if not mso.existing:
            if display_name is None:
                display_name = entry
            if ethertype is None:
                ethertype = "unspecified"
            if ip_protocol is None:
                ip_protocol = "unspecified"
            if tcp_session_rules is None:
                tcp_session_rules = ["unspecified"]
            if source_from is None:
                source_from = "unspecified"
            if source_to is None:
                source_to = "unspecified"
            if destination_from is None:
                destination_from = "unspecified"
            if destination_to is None:
                destination_to = "unspecified"
            if arp_flag is None:
                arp_flag = "unspecified"
            if stateful is None:
                stateful = False
            if fragments_only is None:
                fragments_only = False

        payload = dict(
            name=entry,
            displayName=display_name,
            description=filter_entry_description,
            etherType=ethertype,
            ipProtocol=ip_protocol,
            tcpSessionRules=tcp_session_rules,
            sourceFrom=source_from,
            sourceTo=source_to,
            destinationFrom=destination_from,
            destinationTo=destination_to,
            arpFlag=arp_flag,
            stateful=stateful,
            matchOnlyFragments=fragments_only,
        )

        mso.sanitize(payload, collate=True)

        if filter_idx is None:
            # Filter does not exist, so we have to create it
            if filter_display_name is None:
                filter_display_name = filter_name

            payload = dict(
                name=filter_name,
                displayName=filter_display_name,
                description=filter_description,
                entries=[mso.sent],
            )

            ops.append(dict(op="add", path=filters_path + "/-", value=payload))

        elif entry_idx is None:
            # Entry does not exist, so we have to add it
            ops.append(dict(op="add", path=entries_path + "/-", value=mso.sent))

        else:
            # Entry exists, we have to update it
            for key, value in mso.sent.items():
                ops.append(dict(op="replace", path=entry_path + "/" + key, value=value))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_switch_routing_ospf
short_description: Resource module for networks _switch _routing _ospf
description:
- Manage operation update of the resource networks _switch _routing _ospf.
- Update layer 3 OSPF routing configuration.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  areas:
    description: OSPF areas.
    elements: dict
    suboptions:
      areaId:
        description: OSPF area ID.
        type: str
      areaName:
        description: Name of the OSPF area.
        type: str
      areaType:
        description: Area types in OSPF. Must be one of "normal", "stub", "nssa".
        type: str
    type: list
  deadTimerInSeconds:
    description: Time interval to determine when the peer will be declared inactive/dead.
      Value must be between 1 and 65535.
    type: int
  enabled:
    description: Boolean value to enable or disable OSPF routing. OSPF routing is disabled
      by default.
    type: bool
  helloTimerInSeconds:
    description: Time interval in seconds at which hello packet will be sent to OSPF
      neighbors to maintain connectivity. Value must be between 1 and 255. Default is
      10 seconds.
    type: int
  md5AuthenticationEnabled:
    description: Boolean value to enable or disable MD5 authentication. MD5 authentication
      is disabled by default.
    type: bool
  md5AuthenticationKey:
    description: MD5 authentication credentials. This param is only relevant if md5AuthenticationEnabled
      is true.
    suboptions:
      id:
        description: MD5 authentication key index. Key index must be between 1 to 255.
        type: int
      passphrase:
        description: MD5 authentication passphrase.
        type: str
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  v3:
    description: OSPF v3 configuration.
    suboptions:
      areas:
        description: OSPF v3 areas.
        elements: dict
        suboptions:
          areaId:
            description: OSPF area ID.
            type: str
          areaName:
            description: Name of the OSPF area.
            type: str
          areaType:
            description: Area types in OSPF. Must be one of "normal", "stub", "nssa".
            type: str
        type: list
      deadTimerInSeconds:
        description: Time interval to determine when the peer will be declared inactive/dead.
          Value must be between 1 and 65535.
        type: int
      enabled:
        description: Boolean value to enable or disable V3 OSPF routing. OSPF V3 routing
          is disabled by default.
        type: bool
      helloTimerInSeconds:
        description: Time interval in seconds at which hello packet will be sent to
          OSPF neighbors to maintain connectivity. Value must be between 1 and 255.
          Default is 10 seconds.
        type: int
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch updateNetworkSwitchRoutingOspf
  description: Complete reference of the updateNetworkSwitchRoutingOspf API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-routing-ospf
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_routing_ospf,

  - Paths used are
    put /networks/{networkId}/switch/routing/ospf,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_routing_ospf:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    areas:
    - areaId: '1284392014819'
      areaName: Backbone
      areaType: normal
    deadTimerInSeconds: 40
    enabled: true
    helloTimerInSeconds: 10
    md5AuthenticationEnabled: true
    md5AuthenticationKey:
      id: 1234
      passphrase: abc1234
    networkId: string
    v3:
      areas:
      - areaId: '1284392014819'
        areaName: V3 Backbone
        areaType: normal
      deadTimerInSeconds: 40
      enabled: true
      helloTimerInSeconds: 10

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Route_maps parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_set_extcomm_rt(data):
    cmd = "set extcommunity rt"
    extcomm_numbers = " ".join(data.get("extcommunity_numbers", []))
    if extcomm_numbers:
        cmd += " " + extcomm_numbers
    if data.get("additive"):
        cmd += " additive"

    return cmd


def _tmplt_match_ip_multicast(data):
    cmd = "match ip multicast"
    multicast = data["match"]["ip"]["multicast"]

    if "source" in multicast:
        cmd += " source {source}".format(**multicast)

    if "prefix" in multicast.get("group", {}):
        cmd += " group {prefix}".format(**multicast["group"])
    else:
        if "first" in multicast.get("group_range", {}):
            cmd += " group-range {first}".format(**multicast["group_range"])
        if "last" in multicast.get("group_range", {}):
            cmd += " to {last}".format(**multicast["group_range"])

    if "rp" in multicast:
        cmd += " rp {prefix}".format(**multicast["rp"])
        if "rp_type" in multicast["rp"]:
            cmd += " rp-type {rp_type}".format(**multicast["rp"])

    return cmd


def _tmplt_match_ipv6_multicast(data):
    cmd = "match ipv6 multicast"
    multicast = data["match"]["ipv6"]["multicast"]

    if "source" in multicast:
        cmd += " source {source}".format(**multicast)

    if "prefix" in multicast.get("group", {}):
        cmd += " group {prefix}".format(**multicast["group"])
    else:
        if "first" in multicast.get("group_range", {}):
            cmd += " group-range {first}".format(**multicast["group_range"])
        if "last" in multicast.get("group_range", {}):
            cmd += " to {last}".format(**multicast["group_range"])

    if "rp" in multicast:
        cmd += " rp {prefix}".format(**multicast["rp"])
        if "rp_type" in multicast["rp"]:
            cmd += " rp-type {rp_type}".format(**multicast["rp"])

    return cmd


def _tmplt_set_metric(data):
    cmd = "set metric"
    metric = data["set"]["metric"]

    for x in [
        "bandwidth",
        "igrp_delay_metric",
        "igrp_reliability_metric",
        "igrp_effective_bandwidth_metric",
        "igrp_mtu",
    ]:
        if x in metric:
            cmd += " {0}".format(metric[x])

    return cmd


def _tmplt_set_ip_next_hop_verify_availability(data):
    cmd = []
    for each in data["set"]["ip"]["next_hop"]["verify_availability"]:
        cmd_tmpl = "set ip next-hop verify-availability"
        cmd_tmpl += " {address} track {track}".format(**each)
        if "load_share" in each and each["load_share"]:
            cmd_tmpl += " load-share"
        if "force_order" in each and each["force_order"]:
            cmd_tmpl += " force-order"
        if "drop_on_fail" in each and each["drop_on_fail"]:
            cmd_tmpl += " drop-on-fail"
        cmd.append(cmd_tmpl)
    return cmd


class Route_mapsTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Route_mapsTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "route_map",
            "getval": re.compile(
                r"""
                ^route-map\s(?P<route_map>\S+)\s(?P<action>\S+)\s(?P<sequence>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "route-map {{ route_map }}"
                      "{{ ' ' + action if action is defined else '' }}"
                      "{{ ' ' + sequence|string if sequence is defined else '' }}",
            "result": {
                "{{ route_map }}": {
                    "route_map": "{{ route_map }}",
                    "entries": {
                        "{{ sequence }}": {
                            "sequence": "{{ sequence }}",
                            "action": "{{ action }}",
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "continue_sequence",
            "getval": re.compile(
                r"""
                \s+continue\s(?P<continue_sequence>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "continue {{ continue_sequence }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "continue_sequence": "{{ continue_sequence }}",
                        },
                    },
                },
            },
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                \s+description\s(?P<description>.+)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "description": "{{ description }}",
                        },
                    },
                },
            },
        },
        {
            "name": "match.as_number.asn",
            "getval": re.compile(
                r"""
                \s+match\sas-number
                (?!\sas-path-list)
                \s(?P<asn>.+)\s*
                $""", re.VERBOSE,
            ),
            "setval": "match as-number {{ match.as_number.asn|join(', ') }}",
            "result": {
                "{{ route_map }}": {
                    "route_map": "{{ route_map }}",
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "as_number": {
                                    "asn": "{{ asn.rstrip().split(', ') }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.as_number.as_path_list",
            "getval": re.compile(
                r"""
                \s+match\sas-number
                \sas-path-list\s(?P<as_path_list>.+)\s*
                $""", re.VERBOSE,
            ),
            "setval": "match as-number as-path-list {{ match.as_number.as_path_list|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "as_number": {
                                    "as_path_list": "{{ as_path_list.split() }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.as_path",
            "getval": re.compile(
                r"""
                \s+match\sas-path\s(?P<as_path>.+)\s*
                $""", re.VERBOSE,
            ),
            "setval": "match as-path {{ match.as_path|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "as_path": "{{ as_path.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.community.community_list",
            "getval": re.compile(
                r"""
                \s+match\scommunity
                \s(?P<community_list>.+)
                (\s(?P<exact_match>exact-match))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match community {{ match.community.community_list|join(' ') }}{{ ' exact-match' if match.community.exact_match|d(False) else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "community": {
                                    "community_list": "{{ community_list.split() }}",
                                    "exact_match": "{{ not not exact_match }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.evpn.route_types",
            "getval": re.compile(
                r"""
                \s+match\sevpn
                \sroute-type
                \s(?P<route_types>.+)\s*
                $""", re.VERBOSE,
            ),
            "setval": "match evpn route-type {{ match.evpn.route_types|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "evpn": {
                                    "route_types": "{{ route_types.split() }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.extcommunity.extcommunity_list",
            "getval": re.compile(
                r"""
                \s+match\sextcommunity
                \s(?P<extcommunity_list>.+)
                \s(?P<exact_match>exact-match)?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match extcommunity {{ match.extcommunity.extcommunity_list|join(' ') }}"
                      "{{ ' exact-match' if match.extcommunity.exact_match|d(False) else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "extcommunity": {
                                    "extcommunity_list": "{{ extcommunity_list.split() }}",
                                    "exact_match": "{{ not not exact_match }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.interfaces",
            "getval": re.compile(
                r"""
                \s+match\sinterface
                \s(?P<interfaces>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match interface {{ match.interfaces|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "interfaces": "{{ interfaces.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ip.address.access_list",
            "getval": re.compile(
                r"""
                \s+match\sip\saddress
                \s(?P<access_list>\S+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ip address {{ match.ip.address.access_list }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ip": {
                                    "address": {
                                        "access_list": "{{ access_list }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ip.address.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sip\saddress
                \sprefix-list
                \s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ip address prefix-list {{ match.ip.address.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ip": {
                                    "address": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        # match ip multicast source 192.1.2.0/24 group-range 239.0.0.1 to 239.255.255.255 rp 209.165.201.0/27 rp-type Bidir
        {
            "name": "match.ip.multicast",
            "getval": re.compile(
                r"""
                \s+match\sip\smulticast
                (\ssource\s(?P<source>\S+))?
                (\sgroup\s(?P<prefix>\S+))?
                (\sgroup-range
                (\s(?P<first>\S+))?
                (\sto)?
                (\s(?P<last>\S+)))?
                (\srp\s(?P<rp>\S+))?
                (\srp-type\s(?P<rp_type>\S+))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_match_ip_multicast,
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ip": {
                                    "multicast": {
                                        "group": {
                                            "prefix": "{{ prefix }}",
                                        },
                                        "group_range": {
                                            "first": "{{ first }}",
                                            "last": "{{ last }}",
                                        },
                                        "rp": {
                                            "prefix": "{{ rp }}",
                                            "rp_type": "{{ rp_type }}",
                                        },
                                        "source": "{{ source }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ip.next_hop.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sip\snext-hop
                \sprefix-list\s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ip next-hop prefix-list {{ match.ip.next_hop.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ip": {
                                    "next_hop": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ip.route_source.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sip\sroute-source
                \sprefix-list\s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ip route-source prefix-list {{ match.ip.route_source.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ip": {
                                    "route_source": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ipv6.address.access_list",
            "getval": re.compile(
                r"""
                \s+match\sipv6\saddress
                \s(?P<access_list>\S+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ipv6 address {{ match.ipv6.address.access_list }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ipv6": {
                                    "address": {
                                        "access_list": "{{ access_list }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ipv6.address.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sipv6\saddress
                \sprefix-list
                \s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ipv6 address prefix-list {{ match.ipv6.address.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ipv6": {
                                    "address": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ipv6.multicast",
            "getval": re.compile(
                r"""
                \s+match\sipv6\smulticast
                (\ssource\s(?P<source>\S+))?
                (\sgroup\s(?P<prefix>\S+))?
                (\sgroup-range
                (\s(?P<first>\S+))?
                (\sto)?
                (\s(?P<last>\S+)))?
                (\srp\s(?P<rp>\S+))?
                (\srp-type\s(?P<rp_type>\S+))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_match_ipv6_multicast,
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ipv6": {
                                    "multicast": {
                                        "group": {
                                            "prefix": "{{ prefix }}",
                                        },
                                        "group_range": {
                                            "first": "{{ first }}",
                                            "last": "{{ last }}",
                                        },
                                        "rp": {
                                            "prefix": "{{ rp }}",
                                            "rp_type": "{{ rp_type }}",
                                        },
                                        "source": "{{ source }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ipv6.next_hop.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sipv6\snext-hop
                \sprefix-list\s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ipv6 next-hop prefix-list {{ match.ipv6.next_hop.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ipv6": {
                                    "next_hop": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ipv6.route_source.prefix_lists",
            "getval": re.compile(
                r"""
                \s+match\sipv6\sroute-source
                \sprefix-list\s(?P<prefix_lists>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ipv6 route-source prefix-list {{ match.ipv6.route_source.prefix_lists|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ipv6": {
                                    "route_source": {
                                        "prefix_lists": "{{ prefix_lists.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.mac_list",
            "getval": re.compile(
                r"""
                \s+match\smac-list
                \s(?P<mac_list>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match mac-list {{ match.mac_list|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "mac_list": "{{ mac_list.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.metric",
            "getval": re.compile(
                r"""
                \s+match\smetric
                \s(?P<metric>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match metric {{ match.metric|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "metric": "{{ metric.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.ospf_area",
            "getval": re.compile(
                r"""
                \s+match\sospf-area
                \s(?P<ospf_area>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match ospf-area {{ match.ospf_area|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "ospf_area": "{{ ospf_area.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.route_types",
            "getval": re.compile(
                r"""
                \s+match\sroute-type
                \s(?P<route_types>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match route-type {{ match.route_types|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "route_types": "{{ route_types.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.source_protocol",
            "getval": re.compile(
                r"""
                \s+match\ssource-protocol
                \s(?P<route_type>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match source-protocol {{ match.source_protocol|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "source_protocol": "{{ source_protocol.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "match.tags",
            "getval": re.compile(
                r"""
                \s+match\stag
                \s(?P<tags>.+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "match tag {{ match.tags|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "match": {
                                "tags": "{{ tags.split() }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.as_path.prepend.as_number",
            "getval": re.compile(
                r"""
                \s+set\sas-path\sprepend
                \s(?P<as_number>(?!last-as).+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set as-path prepend {{ set.as_path.prepend.as_number|join(' ') }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "as_path": {
                                    "prepend": {
                                        "as_number": "{{ as_number.split() }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.as_path.prepend.last_as",
            "getval": re.compile(
                r"""
                \s+set\sas-path\sprepend
                \slast-as\s(?P<last_as>\d+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set as-path prepend last-as {{ set.as_path.prepend.last_as|string }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "as_path": {
                                    "prepend": {
                                        "last_as": "{{ last_as }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.as_path.tag",
            "getval": re.compile(
                r"""
                \s+set\sas-path
                \s(?P<tag>tag)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set as-path tag",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "as_path": {
                                    "tag": "{{ not not tag }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.comm_list",
            "getval": re.compile(
                r"""
                \s+set\scomm-list
                \s(?P<comm_list>\S+)
                \s*delete
                \s*$""", re.VERBOSE,
            ),
            "setval": "set comm-list {{ set.comm_list }} delete",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "comm_list": "{{ comm_list }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.community",
            "getval": re.compile(
                r"""
                \s+set\scommunity
                (\s(?P<internet>internet))?
                (?P<number>(\s\d+:\d+)*)
                (\s(?P<no_export>no-export))?
                (\s(?P<no_advertise>no-advertise))?
                (\s(?P<local_as>local-AS))?
                (\s(?P<graceful_shutdown>graceful-shutdown))?
                (\s(?P<additive>additive))?\s*
                $""", re.VERBOSE,
            ),
            "setval": "set community"
                      "{{ ' internet' if set.community.internet|d(False) else '' }}"
                      "{{ ' ' + set.community.number|join(' ') if set.community.number|d(False) else '' }}"
                      "{{ ' no-export' if set.community.no_export|d(False) else '' }}"
                      "{{ ' no-advertise' if set.community.no_advertise|d(False) else '' }}"
                      "{{ ' local-AS' if set.community.local_as|d(False) else '' }}"
                      "{{ ' graceful-shutdown' if set.community.graceful_shutdown|d(False) else '' }}"
                      "{{ ' additive' if set.community.additive|d(False) else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "community": {
                                    "internet": "{{ not not internet }}",
                                    "number": "{{ number.split() }}",
                                    "no_export": "{{ not not no_export }}",
                                    "no_advertise": "{{ not not no_advertise }}",
                                    "local_as": "{{ not not local_as }}",
                                    "graceful_shutdown": "{{ not not graceful_shutdown }}",
                                    "additive": "{{ not not additive }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.dampening",
            "getval": re.compile(
                r"""
                \s+set\sdampening
                \s(?P<half_life>\d+)
                \s(?P<start_reuse_route>\d+)
                \s(?P<start_suppress_route>\d+)
                \s(?P<max_suppress_time>\d+)
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set dampening {{ set.dampening.half_life }}"
                      " {{ set.dampening.start_reuse_route }}"
                      " {{ set.dampening.start_suppress_route }}"
                      " {{ set.dampening.max_suppress_time }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "dampening": {
                                    "half_life": "{{ half_life }}",
                                    "start_reuse_route": "{{ start_reuse_route }}",
                                    "start_suppress_route": "{{ start_suppress_route }}",
                                    "max_suppress_time": "{{ max_suppress_time }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.distance",
            "getval": re.compile(
                r"""
                \s+set\sdistance
                \s(?P<igp_ebgp_routes>\d+)
                (\s(?P<internal_routes>\d+))?
                (\s(?P<local_routes>\d+))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set distance {{ set.distance.igp_ebgp_routes }}"
                      "{{ ' ' + set.distance.internal_routes|string if set.distance.internal_routes|d(False) else '' }}"
                      "{{ ' ' + set.distance.local_routes|string if set.distance.internal_routes|d(False) else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "distance": {
                                    "igp_ebgp_routes": "{{ igp_ebgp_routes }}",
                                    "internal_routes": "{{ internal_routes }}",
                                    "local_routes": "{{ local_routes }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.evpn.gateway_ip",
            "getval": re.compile(
                r"""
                \s+set\sevpn
                \sgateway-ip
                (\s(?P<ip>(?!use-nexthop)\S+))?
                (\s(?P<use_nexthop>use-nexthop))?
                \s*
                $""", re.VERBOSE,
            ),
            "setval": "set evpn gateway-ip"
                      "{{ ' ' + set.evpn.gateway_ip.ip if set.evpn.gateway_ip.ip|d(False) else ''}}"
                      "{{ ' use-nexthop' if set.evpn.gateway_ip.use_nexthop|d(False) else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "evpn": {
                                    "gateway_ip": {
                                        "ip": "{{ ip }}",
                                        "use_nexthop": "{{ not not use_nexthop }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.extcomm_list",
            "getval": re.compile(
                r"""
                \s+set\sextcomm-list
                \s(?P<extcomm_list>\S+)
                \s*delete
                \s*$""", re.VERBOSE,
            ),
            "setval": "set extcomm-list {{ set.extcomm_list }} delete",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "extcomm_list": "{{ extcomm_list }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.extcommunity.rt",
            "getval": re.compile(
                r"""
                \s+set\sextcommunity\srt
                (?P<extcommunity_numbers>(\s\S+:\S+)*)?
                (\s(?P<additive>additive))?
                \s*$""", re.VERBOSE,
            ),
            "setval": _tmplt_set_extcomm_rt,
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "extcommunity": {
                                    "rt": {
                                        "additive": "{{ not not additive }}",
                                        "extcommunity_numbers":
                                            "{{ extcommunity_numbers.strip().split(' ') if extcommunity_numbers|d('') else None }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.forwarding_address",
            "getval": re.compile(
                r"""
                \s+set
                \s(?P<forwarding_address>forwarding-address)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set forwarding-address",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "forwarding_address": "{{ not not forwarding_address }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.null_interface",
            "getval": re.compile(
                r"""
                \s+set\sinterface
                \s(?P<interface>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set interface {{ set.null_interface }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "null_interface": "{{ interface }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.address.prefix_list",
            "getval": re.compile(
                r"""
                \s+set\sip\saddress
                \sprefix-list\s(?P<prefix_list>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip address prefix-list {{ set.ip.address.prefix_list }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "address": {
                                        "prefix_list": "{{ prefix_list }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.precedence",
            "getval": re.compile(
                r"""
                \s+set\sip
                \sprecedence\s(?P<precedence>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip precedence {{ set.ip.precedence }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "precedence": "{{ precedence }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.next_hop",
            "getval": re.compile(
                r"""
                \s+set\sip\snext-hop
                \s(?P<address>(\s?((25[0-5]|(2[0-4]|1\d|[1-9]|)\d)\.?\b){4})+)
                (\s+(?P<load_share>load-share))?
                (\s+(?P<force_order>force-order))?
                (\s+(?P<drop_on_fail>drop-on-fail))?
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip next-hop {{ set.ip.next_hop.address }}"
                      "{{ ' load-share' if set.ip.next_hop.load_share else '' }}"
                      "{{ ' force-order' if set.ip.next_hop.force_order else '' }}"
                      "{{ ' drop-on-fail' if set.ip.next_hop.drop_on_fail else '' }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "next_hop": {
                                        "address": "{{ address }}",
                                        "load_share": "{{ not not load_share|d(False) }}",
                                        "force_order": "{{ not not force_order|d(False) }}",
                                        "drop_on_fail": "{{ not not drop_on_fail|d(False) }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.next_hop.peer_address",
            "getval": re.compile(
                r"""
                \s+set\sip\snext-hop
                \s(?P<peer_address>peer-address)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip next-hop peer-address",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "next_hop": {
                                        "peer_address": "{{ not not peer_address }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.next_hop.redist_unchanged",
            "getval": re.compile(
                r"""
                \s+set\sip\snext-hop
                \s(?P<redist_unchanged>redist-unchanged)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip next-hop redist-unchanged",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "next_hop": {
                                        "redist_unchanged": "{{ not not redist_unchanged }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.next_hop.unchanged",
            "getval": re.compile(
                r"""
                \s+set\sip\snext-hop
                \s(?P<unchanged>unchanged)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ip next-hop unchanged",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "next_hop": {
                                        "unchanged": "{{ not not unchanged }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ip.next_hop.verify_availability",
            "getval": re.compile(
                r"""
                \s+set\sip\snext-hop\sverify-availability
                \s(?P<address>\S+)
                \strack\s(?P<track>\d)
                (\s(?P<load_share>load-share))?
                (\s(?P<force_order>force-order))?
                (\s(?P<drop_on_fail>drop-on-fail))?
                \s*$""", re.VERBOSE,
            ),
            "setval": _tmplt_set_ip_next_hop_verify_availability,
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ip": {
                                    "next_hop": {
                                        "verify_availability": [
                                            {
                                                "address": "{{ address }}",
                                                "track": "{{ track }}",
                                                "load_share": "{{ not not load_share|d(False) }}",
                                                "force_order": "{{ not not force_order|d(False) }}",
                                                "drop_on_fail": "{{ not not drop_on_fail|d(False) }}",
                                            },
                                        ],
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ipv6.address.prefix_list",
            "getval": re.compile(
                r"""
                \s+set\sipv6\saddress
                \sprefix-list\s(?P<prefix_list>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ipv6 address prefix-list {{ set.ipv6.address.prefix_list }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ipv6": {
                                    "address": {
                                        "prefix_list": "{{ prefix_list }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.ipv6.precedence",
            "getval": re.compile(
                r"""
                \s+set\sipv6
                \sprecedence\s(?P<precedence>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set ipv6 precedence {{ set.ipv6.precedence }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "ipv6": {
                                    "precedence": "{{ precedence }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.label_index",
            "getval": re.compile(
                r"""
                \s+set\slabel-index
                \s(?P<label_index>\d+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set label-index {{ set.label_index }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "label_index": "{{ label_index }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.level",
            "getval": re.compile(
                r"""
                \s+set\slevel
                \s(?P<level>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set level {{ set.level }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "level": "{{ level }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.local_preference",
            "getval": re.compile(
                r"""
                \s+set\slocal-preference
                \s(?P<local_preference>\d+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set local-preference {{ set.local_preference }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "local_preference": "{{ local_preference }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.metric",
            "getval": re.compile(
                r"""
                \s+set\smetric
                \s(?P<bandwidth>\d+)
                (\s(?P<igrp_delay_metric>\d+))?
                (\s(?P<igrp_reliability_metric>\d+))?
                (\s(?P<igrp_effective_bandwidth_metric>\d+))?
                (\s(?P<igrp_mtu>\d+))?
                \s*$""", re.VERBOSE,
            ),
            "setval": _tmplt_set_metric,
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "metric": {
                                    "bandwidth": "{{ bandwidth }}",
                                    "igrp_delay_metric": "{{ igrp_delay_metric }}",
                                    "igrp_reliability_metric": "{{ igrp_reliability_metric }}",
                                    "igrp_effective_bandwidth_metric": "{{ igrp_effective_bandwidth_metric }}",
                                    "igrp_mtu": "{{ igrp_mtu }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.metric_type",
            "getval": re.compile(
                r"""
                \s+set\smetric-type
                \s(?P<metric_type>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set metric-type {{ set.metric_type }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "metric_type": "{{ metric_type }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.nssa_only",
            "getval": re.compile(
                r"""
                \s+set
                \s(?P<nssa_only>nssa-only)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set nssa-only",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "nssa_only": "{{ not not nssa_only }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.origin",
            "getval": re.compile(
                r"""
                \s+set\sorigin
                \s(?P<origin>\S+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set origin {{ set.origin }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "origin": "{{ origin }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.path_selection",
            "getval": re.compile(
                r"""
                \s+set\spath-selection
                \s(?P<path_selection>\S+)
                \sadvertise
                \s*$""", re.VERBOSE,
            ),
            "setval": "set path-selection {{ set.path_selection }} advertise",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "path_selection": "{{ path_selection }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.tag",
            "getval": re.compile(
                r"""
                \s+set\stag
                \s(?P<tag>\d+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set tag {{ set.tag }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "tag": "{{ tag }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "set.weight",
            "getval": re.compile(
                r"""
                \s+set\sweight
                \s(?P<weight>\d+)
                \s*$""", re.VERBOSE,
            ),
            "setval": "set weight {{ set.weight }}",
            "result": {
                "{{ route_map }}": {
                    "entries": {
                        "{{ sequence }}": {
                            "set": {
                                "weight": "{{ weight }}",
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



# -*- coding: utf-8 -*-

# Copyright (c) 2019 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Author:
#   - Matthew Davis <Matthew.Davis.2@team.telstra.com>
#     on behalf of Telstra Corporation Limited
#
# Common functionality to be used by the modules:
#   - acm_certificate
#   - acm_certificate_info

"""
Common Amazon Certificate Manager facts shared between modules
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass

from ansible.module_utils._text import to_bytes
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from .botocore import is_boto3_error_code
from .retries import AWSRetry
from .tagging import ansible_dict_to_boto3_tag_list
from .tagging import boto3_tag_list_to_ansible_dict


def acm_catch_boto_exception(func):
    def runner(*args, **kwargs):
        module = kwargs.pop("module", None)
        error = kwargs.pop("error", None)
        ignore_error_codes = kwargs.pop("ignore_error_codes", [])

        try:
            return func(*args, **kwargs)
        except is_boto3_error_code(ignore_error_codes):
            return None
        except (BotoCoreError, ClientError) as e:
            if not module:
                raise
            module.fail_json_aws(e, msg=error)

    return runner


class ACMServiceManager:
    """Handles ACM Facts Services"""

    def __init__(self, module):
        self.module = module
        self.client = module.client("acm")

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(delay=5, catch_extra_error_codes=["RequestInProgressException"])
    def delete_certificate_with_backoff(self, arn):
        self.client.delete_certificate(CertificateArn=arn)

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(delay=5, catch_extra_error_codes=["RequestInProgressException"])
    def list_certificates_with_backoff(self, statuses=None):
        paginator = self.client.get_paginator("list_certificates")
        # `list_certificates` requires explicit key type filter, or it returns only RSA_2048 certificates
        kwargs = {
            "Includes": {
                "keyTypes": [
                    "RSA_1024",
                    "RSA_2048",
                    "RSA_3072",
                    "RSA_4096",
                    "EC_prime256v1",
                    "EC_secp384r1",
                    "EC_secp521r1",
                ],
            },
        }
        if statuses:
            kwargs["CertificateStatuses"] = statuses
        return paginator.paginate(**kwargs).build_full_result()["CertificateSummaryList"]

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(
        delay=5, catch_extra_error_codes=["RequestInProgressException", "ResourceNotFoundException"]
    )
    def get_certificate_with_backoff(self, certificate_arn):
        response = self.client.get_certificate(CertificateArn=certificate_arn)
        # strip out response metadata
        return {"Certificate": response["Certificate"], "CertificateChain": response["CertificateChain"]}

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(
        delay=5, catch_extra_error_codes=["RequestInProgressException", "ResourceNotFoundException"]
    )
    def describe_certificate_with_backoff(self, certificate_arn):
        return self.client.describe_certificate(CertificateArn=certificate_arn)["Certificate"]

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(
        delay=5, catch_extra_error_codes=["RequestInProgressException", "ResourceNotFoundException"]
    )
    def list_certificate_tags_with_backoff(self, certificate_arn):
        return self.client.list_tags_for_certificate(CertificateArn=certificate_arn)["Tags"]

    @acm_catch_boto_exception
    @AWSRetry.jittered_backoff(delay=5, catch_extra_error_codes=["RequestInProgressException"])
    def import_certificate_with_backoff(self, certificate, private_key, certificate_chain, arn):
        params = {"Certificate": to_bytes(certificate), "PrivateKey": to_bytes(private_key)}
        if arn:
            params["CertificateArn"] = arn
        if certificate_chain:
            params["CertificateChain"] = certificate_chain

        return self.client.import_certificate(**params)["CertificateArn"]

    # Tags are a normal Ansible style dict
    # {'Key':'Value'}
    @AWSRetry.jittered_backoff(
        delay=5, catch_extra_error_codes=["RequestInProgressException", "ResourceNotFoundException"]
    )
    def tag_certificate_with_backoff(self, arn, tags):
        aws_tags = ansible_dict_to_boto3_tag_list(tags)
        self.client.add_tags_to_certificate(CertificateArn=arn, Tags=aws_tags)

    def _match_tags(self, ref_tags, cert_tags):
        if ref_tags is None:
            return True
        try:
            return all(k in cert_tags for k in ref_tags) and all(cert_tags.get(k) == ref_tags[k] for k in ref_tags)
        except (TypeError, AttributeError) as e:
            self.module.fail_json_aws(e, msg="ACM tag filtering err")

    def delete_certificate(self, *args, arn=None):
        # hacking for backward compatibility
        if arn is None:
            if len(args) < 3:
                self.module.fail_json(msg="Missing required certificate arn to delete.")
            arn = args[2]
        error = f"Couldn't delete certificate {arn}"
        self.delete_certificate_with_backoff(arn, module=self.module, error=error)

    def get_certificates(self, *args, domain_name=None, statuses=None, arn=None, only_tags=None, **kwargs):
        """
        Returns a list of certificates
        if domain_name is specified, returns only certificates with that domain
        if an ARN is specified, returns only that certificate
        only_tags is a dict, e.g. {'key':'value'}. If specified this function will return
        only certificates which contain all those tags (key exists, value matches).
        """
        all_certificates = self.list_certificates_with_backoff(
            statuses=statuses, module=self.module, error="Couldn't obtain certificates"
        )

        def _filter_certificate(cert):
            if domain_name and cert["DomainName"] != domain_name:
                return False
            if arn and cert["CertificateArn"] != arn:
                return False
            return True

        certificates = list(filter(_filter_certificate, all_certificates))

        results = []
        for certificate in certificates:
            cert_data = self.describe_certificate_with_backoff(
                certificate["CertificateArn"],
                module=self.module,
                error=f"Couldn't obtain certificate metadata for domain {certificate['DomainName']}",
                ignore_error_codes=["ResourceNotFoundException"],
            )
            if cert_data is None:
                continue

            # in some states, ACM resources do not have a corresponding cert
            if cert_data["Status"] not in ("PENDING_VALIDATION", "VALIDATION_TIMED_OUT", "FAILED"):
                cert_info = self.get_certificate_with_backoff(
                    certificate["CertificateArn"],
                    module=self.module,
                    error=f"Couldn't obtain certificate data for domain {certificate['DomainName']}",
                    ignore_error_codes=["ResourceNotFoundException"],
                )
                if cert_info is None:
                    continue
                cert_data.update(cert_info)

            cert_data = camel_dict_to_snake_dict(cert_data)
            tags = self.list_certificate_tags_with_backoff(
                certificate["CertificateArn"],
                module=self.module,
                error=f"Couldn't obtain tags for domain {certificate['DomainName']}",
                ignore_error_codes=["ResourceNotFoundException"],
            )
            if tags is None:
                continue

            tags = boto3_tag_list_to_ansible_dict(tags)
            if not self._match_tags(only_tags, tags):
                continue
            cert_data["tags"] = tags
            results.append(cert_data)
        return results

    def get_domain_of_cert(self, arn, **kwargs):
        """
        returns the domain name of a certificate (encoded in the public cert)
        for a given ARN A cert with that ARN must already exist
        """
        if arn is None:
            self.module.fail_json(msg="Internal error with ACM domain fetching, no certificate ARN specified")
        error = f"Couldn't obtain certificate data for arn {arn}"
        cert_data = self.describe_certificate_with_backoff(certificate_arn=arn, module=self.module, error=error)
        return cert_data["DomainName"]

    def import_certificate(self, *args, certificate, private_key, arn=None, certificate_chain=None, tags=None):
        original_arn = arn

        # upload cert
        params = {
            "certificate": certificate,
            "private_key": private_key,
            "certificate_chain": certificate_chain,
            "arn": arn,
            "module": self.module,
            "error": "Couldn't upload new certificate",
        }
        arn = self.import_certificate_with_backoff(**params)
        if original_arn and (arn != original_arn):
            # I'm not sure whether the API guarentees that the ARN will not change
            # I'm failing just in case.
            # If I'm wrong, I'll catch it in the integration tests.
            self.module.fail_json(msg=f"ARN changed with ACM update, from {original_arn} to {arn}")

        # tag that cert
        try:
            self.tag_certificate_with_backoff(arn, tags)
        except (BotoCoreError, ClientError) as e:
            try:
                self.delete_certificate_with_backoff(arn)
            except (BotoCoreError, ClientError):
                self.module.warn(
                    f"Certificate {arn} exists, and is not tagged. So Ansible will not see it on the next run."
                )
                self.module.fail_json_aws(e, msg=f"Couldn't tag certificate {arn}, couldn't delete it either")
            self.module.fail_json_aws(e, msg=f"Couldn't tag certificate {arn}")

        return arn



#!/usr/bin/python
#
# Copyright (c) 2020  haiyuazhang <haiyzhan@micosoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_openshiftmanagedcluster_info
version_added: '1.11.0'
short_description: Get Info onf Azure Red Hat OpenShift Managed Cluster
description:
    - Create, update and delete instance of Azure Red Hat OpenShift Managed Cluster instance.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: false
        type: str
    name:
        description:
            - Resource name.
        required: false
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Paul Czarkowski (@paulczar)
'''

EXAMPLES = '''
- name: List all Azure Red Hat OpenShift Managed Clusters for a given subscription
  azure_rm_openshiftmanagedcluster_info:
- name: List all Azure Red Hat OpenShift Managed Clusters for a given resource group
  azure_rm_openshiftmanagedcluster_info:
    resource_group: myResourceGroup
- name: Get Azure Red Hat OpenShift Managed Clusters
  azure_rm_openshiftmanagedcluster_info:
    resource_group: myResourceGroup
    name: myAzureFirewall
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: /subscriptions/xx-xx-xx-xx/resourceGroups/mycluster-eastus/providers/Microsoft.RedHatOpenShift/openShiftClusters/mycluster
name:
    description:
        - Resource name.
    returned: always
    type: str
    sample: mycluster
type:
    description:
        - Resource type.
    returned: always
    type: str
    sample: Microsoft.RedHatOpenShift/openShiftClusters
location:
    description:
        - Resource location.
    returned: always
    type: str
    sample: eatus
properties:
    description:
        - Properties of a OpenShift managed cluster.
    returned: always
    type: complex
    sample: null
    contains:
        provisioningState:
            description:
                - The current deployment or provisioning state, which only appears in the response.
            returned: always
            type: str
            sample: Creating
        clusterProfile:
            description:
                - Configuration for Openshift cluster.
            returned: always
            type: complex
            contains:
                domain:
                    description:
                        - Domain for the cluster.
                    returned: always
                    type: str
                    sample: mycluster
                version:
                    description:
                        - Openshift version.
                    returned: always
                    type: str
                    sample: 4.4.17
                resourceGroupId:
                    description:
                        - The ID of the cluster resource group.
                    returned: always
                    type: str
                    sample: /subscriptions/xx-xx-xx-xx/resourceGroups/mycluster-eastus-cluster
        servicePrincipalProfile:
            description:
                - Service principal.
            type: complex
            returned: always
            contains:
                clientId:
                    description: Client ID of the service principal.
                    returned: always
                    type: str
                    sample: xxxxxxxx-xxxx-xxxx-xxxxxxxxxxxx
        networkProfile:
            description:
                - Configuration for OpenShift networking.
            returned: always
            type: complex
            contains:
                podCidr:
                    description:
                        - CIDR for the OpenShift Pods.
                    returned: always
                    type: str
                    sample: 10.128.0.0/14
                serviceCidr:
                    description:
                        - CIDR for OpenShift Services.
                    type: str
                    returned: always
                    sample: 172.30.0.0/16
        masterProfile:
            description:
                - Configuration for OpenShift master VMs.
            returned: always
            type: complex
            contains:
                vmSize:
                    description:
                        - Size of agent VMs (immutable).
                    type: str
                    returned: always
                    sample: Standard_D8s_v3
                subnetId:
                    description:
                        - The Azure resource ID of the master subnet (immutable).
                    type: str
                    returned: always
                    sample: /subscriptions/xx-xx-xx-xx/resourceGroups/mycluster-eastus/providers/Microsoft.Network/
                            virtualNetworks/mycluster-vnet/subnets/mycluster-worker
        workerProfiles:
            description:
                - Configuration of OpenShift cluster VMs.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - Unique name of the pool profile in the context of the subscription and resource group.
                    returned: always
                    type: str
                    sample: worker
                count:
                    description:
                        - Number of agents (VMs) to host docker containers.
                    returned: always
                    type: int
                    sample: 3
                vmSize:
                    description:
                        - Size of agent VMs.
                    returned: always
                    type: str
                    sample: Standard_D4s_v3
                diskSizeGB:
                    description:
                        - disk size in GB.
                    returned: always
                    type: int
                    sample: 128
                subnetId:
                    description:
                        - Subnet ID for worker pool.
                    returned: always
                    type: str
                    sample: /subscriptions/xx-xx-xx-xx/resourceGroups/mycluster-eastus/providers/Microsoft.Network/
                            virtualNetworks/mycluster-vnet/subnets/mycluster-worker
        ingressProfiles:
            description:
                - Ingress configruation.
            returned: always
            type: list
            sample: [{"name": "default", "visibility": "Public"}, ]
        apiserverProfile:
            description:
                - API server configuration.
            returned: always
            type: complex
            contains:
                visibility:
                    description:
                        - api server visibility.
                    returned: always
                    type: str
                    sample: Public
'''

import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMOpenShiftManagedClustersInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.name = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2020-04-30'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        self.mgmt_client = None
        super(AzureRMOpenShiftManagedClustersInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    is_track2=True,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if (self.resource_group is not None and self.name is not None):
            self.results['clusters'] = self.get()
        elif (self.resource_group is not None):
            self.results['clusters'] = self.list()
        else:
            self.results['clusters'] = self.listall()
        return self.results

    def get(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.RedHatOpenShift' +
                    '/openShiftClusters' +
                    '/{{ open_shift_managed_cluster_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ open_shift_managed_cluster_name }}', self.name)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')
            return {}

        return self.format_item(results)

    def list(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.RedHatOpenShift' +
                    '/openShiftClusters')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return [self.format_item(x) for x in results['value']] if results['value'] else []

    def listall(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/providers' +
                    '/Microsoft.RedHatOpenShift' +
                    '/openShiftClusters')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
            # self.log('Response : {0}'.format(response))
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')
        return results
        # return [self.format_item(x) for x in results['value']] if results['value'] else []

    def format_item(self, item):
        return item
        # d = {
        #     'id': item['id'],
        #     'name': item['name'],
        #     'location': item['location'],
        #     'tags': item.get('tags'),
        #     'api_server_url': item['properties']['apiserverProfile']['url'],
        #     'console_url': item['properties']['consoleProfile']['url'],
        #     'provisioning_state': item['properties']['provisioningState']
        # }
        # return d


def main():
    AzureRMOpenShiftManagedClustersInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import sys

DEFAULT_DOCKER_API_VERSION = '1.41'
MINIMUM_DOCKER_API_VERSION = '1.21'
DEFAULT_TIMEOUT_SECONDS = 60
STREAM_HEADER_SIZE_BYTES = 8
CONTAINER_LIMITS_KEYS = [
    'memory', 'memswap', 'cpushares', 'cpusetcpus'
]

DEFAULT_HTTP_HOST = "127.0.0.1"
DEFAULT_UNIX_SOCKET = "http+unix:///var/run/docker.sock"
DEFAULT_NPIPE = 'npipe:////./pipe/docker_engine'

BYTE_UNITS = {
    'b': 1,
    'k': 1024,
    'm': 1024 * 1024,
    'g': 1024 * 1024 * 1024
}

IS_WINDOWS_PLATFORM = (sys.platform == 'win32')
WINDOWS_LONGPATH_PREFIX = '\\\\?\\'

DEFAULT_USER_AGENT = "ansible-community.docker"
DEFAULT_NUM_POOLS = 25

# The OpenSSH server default value for MaxSessions is 10 which means we can
# use up to 9, leaving the final session for the underlying SSH connection.
# For more details see: https://github.com/docker/docker-py/issues/2246
DEFAULT_NUM_POOLS_SSH = 9

DEFAULT_MAX_POOL_SIZE = 10

DEFAULT_DATA_CHUNK_SIZE = 1024 * 2048

DEFAULT_SWARM_ADDR_POOL = ['10.0.0.0/8']
DEFAULT_SWARM_SUBNET_SIZE = 24



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_block_storage
short_description: Create/destroy or attach/detach Block Storage volumes in DigitalOcean
description:
     - Create/destroy Block Storage volume in DigitalOcean, or attach/detach Block Storage volume to a droplet.
options:
  command:
    description:
     - Which operation do you want to perform.
    choices: ['create', 'attach']
    required: true
    type: str
  state:
    description:
     - Indicate desired state of the target.
    choices: ['present', 'absent']
    required: true
    type: str
  block_size:
    description:
    - The size of the Block Storage volume in gigabytes.
    - Required when I(command=create) and I(state=present).
    - If snapshot_id is included, this will be ignored.
    - If block_size > current size of the volume, the volume is resized.
    type: int
  volume_name:
    description:
    - The name of the Block Storage volume.
    type: str
    required: true
  description:
    description:
    - Description of the Block Storage volume.
    type: str
  region:
    description:
    - The slug of the region where your Block Storage volume should be located in.
    - If I(snapshot_id) is included, this will be ignored.
    type: str
  snapshot_id:
    description:
    - The snapshot id you would like the Block Storage volume created with.
    - If included, I(region) and I(block_size) will be ignored and changed to C(null).
    type: str
  droplet_id:
    description:
    - The droplet id you want to operate on.
    - Required when I(command=attach).
    type: int
  project_name:
    aliases: ["project"]
    description:
    - Project to assign the resource to (project name, not UUID).
    - Defaults to the default project of the account (empty string).
    - Currently only supported when C(command=create).
    type: str
    required: false
    default: ""
extends_documentation_fragment:
- community.digitalocean.digital_ocean.documentation

notes:
  - Two environment variables can be used, DO_API_KEY and DO_API_TOKEN.
    They both refer to the v2 token.
  - If snapshot_id is used, region and block_size will be ignored and changed to null.

author:
    - "Harnek Sidhu (@harneksidhu)"
"""

EXAMPLES = r"""
- name: Create new Block Storage
  community.digitalocean.digital_ocean_block_storage:
    state: present
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: create
    region: nyc1
    block_size: 10
    volume_name: nyc1-block-storage

- name: Create new Block Storage (and assign to Project "test")
  community.digitalocean.digital_ocean_block_storage:
    state: present
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: create
    region: nyc1
    block_size: 10
    volume_name: nyc1-block-storage
    project_name: test

- name: Resize an existing Block Storage
  community.digitalocean.digital_ocean_block_storage:
    state: present
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: create
    region: nyc1
    block_size: 20
    volume_name: nyc1-block-storage

- name: Delete Block Storage
  community.digitalocean.digital_ocean_block_storage:
    state: absent
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: create
    region: nyc1
    volume_name: nyc1-block-storage

- name: Attach Block Storage to a Droplet
  community.digitalocean.digital_ocean_block_storage:
    state: present
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: attach
    volume_name: nyc1-block-storage
    region: nyc1
    droplet_id: <ID>

- name: Detach Block Storage from a Droplet
  community.digitalocean.digital_ocean_block_storage:
    state: absent
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_TOKEN') }}"
    command: attach
    volume_name: nyc1-block-storage
    region: nyc1
    droplet_id: <ID>
"""

RETURN = r"""
id:
    description: Unique identifier of a Block Storage volume returned during creation.
    returned: changed
    type: str
    sample: "69b25d9a-494c-12e6-a5af-001f53126b44"
msg:
    description: Informational or error message encountered during execution
    returned: changed
    type: str
    sample: No project named test2 found
assign_status:
    description: Assignment status (ok, not_found, assigned, already_assigned, service_down)
    returned: changed
    type: str
    sample: assigned
resources:
    description: Resource assignment involved in project assignment
    returned: changed
    type: dict
    sample:
        assigned_at: '2021-10-25T17:39:38Z'
        links:
            self: https://api.digitalocean.com/v2/volumes/8691c49e-35ba-11ec-9406-0a58ac1472b9
        status: assigned
        urn: do:volume:8691c49e-35ba-11ec-9406-0a58ac1472b9
"""

import time
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
    DigitalOceanProjects,
)


class DOBlockStorageException(Exception):
    pass


class DOBlockStorage(object):
    def __init__(self, module):
        self.module = module
        self.rest = DigitalOceanHelper(module)
        if self.module.params.get("project_name"):
            # only load for non-default project assignments
            self.projects = DigitalOceanProjects(module, self.rest)

    def get_key_or_fail(self, k):
        v = self.module.params[k]
        if v is None:
            self.module.fail_json(msg="Unable to load %s" % k)
        return v

    def poll_action_for_complete_status(self, action_id):
        url = "actions/{0}".format(action_id)
        end_time = time.monotonic() + self.module.params["timeout"]
        while time.monotonic() < end_time:
            time.sleep(10)
            response = self.rest.get(url)
            status = response.status_code
            json = response.json
            if status == 200:
                if json["action"]["status"] == "completed":
                    return True
                elif json["action"]["status"] == "errored":
                    raise DOBlockStorageException(json["message"])
        raise DOBlockStorageException(
            "Unable to reach the DigitalOcean API at %s"
            % self.module.params.get("baseurl")
        )

    def get_block_storage_by_name(self, volume_name, region):
        url = "volumes?name={0}&region={1}".format(volume_name, region)
        resp = self.rest.get(url)
        if resp.status_code != 200:
            raise DOBlockStorageException(resp.json["message"])

        volumes = resp.json["volumes"]
        if not volumes:
            return None

        return volumes[0]

    def get_attached_droplet_ID(self, volume_name, region):
        volume = self.get_block_storage_by_name(volume_name, region)
        if not volume or not volume["droplet_ids"]:
            return None

        return volume["droplet_ids"][0]

    def attach_detach_block_storage(self, method, volume_name, region, droplet_id):
        data = {
            "type": method,
            "volume_name": volume_name,
            "region": region,
            "droplet_id": droplet_id,
        }
        response = self.rest.post("volumes/actions", data=data)
        status = response.status_code
        json = response.json
        if status == 202:
            return self.poll_action_for_complete_status(json["action"]["id"])
        elif status == 200:
            return True
        elif status == 404 and method == "detach":
            return False  # Already detached
        elif status == 422:
            return False
        else:
            raise DOBlockStorageException(json["message"])

    def resize_block_storage(self, volume_name, region, desired_size):
        if not desired_size:
            return False

        volume = self.get_block_storage_by_name(volume_name, region)
        if volume["size_gigabytes"] == desired_size:
            return False

        data = {
            "type": "resize",
            "size_gigabytes": desired_size,
        }
        resp = self.rest.post(
            "volumes/{0}/actions".format(volume["id"]),
            data=data,
        )
        if resp.status_code == 202:
            return self.poll_action_for_complete_status(resp.json["action"]["id"])
        else:
            # we'd get status 422 if desired_size <= current volume size
            raise DOBlockStorageException(resp.json["message"])

    def create_block_storage(self):
        volume_name = self.get_key_or_fail("volume_name")
        snapshot_id = self.module.params["snapshot_id"]
        if snapshot_id:
            self.module.params["block_size"] = None
            self.module.params["region"] = None
            block_size = None
            region = None
        else:
            block_size = self.get_key_or_fail("block_size")
            region = self.get_key_or_fail("region")
        description = self.module.params["description"]
        data = {
            "size_gigabytes": block_size,
            "name": volume_name,
            "description": description,
            "region": region,
            "snapshot_id": snapshot_id,
        }
        response = self.rest.post("volumes", data=data)
        status = response.status_code
        json = response.json
        if status == 201:
            project_name = self.module.params.get("project_name")
            if (
                project_name
            ):  # empty string is the default project, skip project assignment
                urn = "do:volume:{0}".format(json["volume"]["id"])
                (
                    assign_status,
                    error_message,
                    resources,
                ) = self.projects.assign_to_project(project_name, urn)
                self.module.exit_json(
                    changed=True,
                    id=json["volume"]["id"],
                    msg=error_message,
                    assign_status=assign_status,
                    resources=resources,
                )
            else:
                self.module.exit_json(changed=True, id=json["volume"]["id"])
        elif status == 409 and json["id"] == "conflict":
            # The volume exists already, but it might not have the desired size
            resized = self.resize_block_storage(volume_name, region, block_size)
            self.module.exit_json(changed=resized)
        else:
            raise DOBlockStorageException(json["message"])

    def delete_block_storage(self):
        volume_name = self.get_key_or_fail("volume_name")
        region = self.get_key_or_fail("region")
        url = "volumes?name={0}&region={1}".format(volume_name, region)
        attached_droplet_id = self.get_attached_droplet_ID(volume_name, region)
        if attached_droplet_id is not None:
            self.attach_detach_block_storage(
                "detach", volume_name, region, attached_droplet_id
            )
        response = self.rest.delete(url)
        status = response.status_code
        json = response.json
        if status == 204:
            self.module.exit_json(changed=True)
        elif status == 404:
            self.module.exit_json(changed=False)
        else:
            raise DOBlockStorageException(json["message"])

    def attach_block_storage(self):
        volume_name = self.get_key_or_fail("volume_name")
        region = self.get_key_or_fail("region")
        droplet_id = self.get_key_or_fail("droplet_id")
        attached_droplet_id = self.get_attached_droplet_ID(volume_name, region)
        if attached_droplet_id is not None:
            if attached_droplet_id == droplet_id:
                self.module.exit_json(changed=False)
            else:
                self.attach_detach_block_storage(
                    "detach", volume_name, region, attached_droplet_id
                )
        changed_status = self.attach_detach_block_storage(
            "attach", volume_name, region, droplet_id
        )
        self.module.exit_json(changed=changed_status)

    def detach_block_storage(self):
        volume_name = self.get_key_or_fail("volume_name")
        region = self.get_key_or_fail("region")
        droplet_id = self.get_key_or_fail("droplet_id")
        changed_status = self.attach_detach_block_storage(
            "detach", volume_name, region, droplet_id
        )
        self.module.exit_json(changed=changed_status)


def handle_request(module):
    block_storage = DOBlockStorage(module)
    command = module.params["command"]
    state = module.params["state"]
    if command == "create":
        if state == "present":
            block_storage.create_block_storage()
        elif state == "absent":
            block_storage.delete_block_storage()
    elif command == "attach":
        if state == "present":
            block_storage.attach_block_storage()
        elif state == "absent":
            block_storage.detach_block_storage()


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        state=dict(choices=["present", "absent"], required=True),
        command=dict(choices=["create", "attach"], required=True),
        block_size=dict(type="int", required=False),
        volume_name=dict(type="str", required=True),
        description=dict(type="str"),
        region=dict(type="str", required=False),
        snapshot_id=dict(type="str", required=False),
        droplet_id=dict(type="int"),
        project_name=dict(type="str", aliases=["project"], required=False, default=""),
    )

    module = AnsibleModule(argument_spec=argument_spec)

    try:
        handle_request(module)
    except DOBlockStorageException as e:
        module.fail_json(msg=str(e), exception=traceback.format_exc())
    except KeyError as e:
        module.fail_json(msg="Unable to load %s" % e, exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: users_external_servers_info
short_description: Information module for Users External Servers
description:
- Get all Users External Servers.
- Get external users authentication servers.
version_added: '6.7.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  invokeSource:
    description:
    - InvokeSource query parameter. The source that invokes this API.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles GetExternalAuthenticationServersAPI
  description: Complete reference of the GetExternalAuthenticationServersAPI API.
  link: https://developer.cisco.com/docs/dna-center/#!get-external-authentication-servers-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.get_external_authentication_servers_ap_i,

  - Paths used are
    get /dna/system/api/v1/users/external-servers,

"""

EXAMPLES = r"""
- name: Get all Users External Servers
  cisco.dnac.users_external_servers_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    invokeSource: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "aaa-servers": [
        {
          "accountingPort": 0,
          "retries": 0,
          "protocol": "string",
          "socketTimeout": 0,
          "serverIp": "string",
          "sharedSecret": "string",
          "serverId": "string",
          "authenticationPort": 0,
          "aaaAttribute": "string",
          "role": "string"
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lan_automation_log_info
short_description: Information module for Lan Automation Log
description:
- Get all Lan Automation Log.
- Get Lan Automation Log by id.
- Invoke this API to get the LAN Automation session logs based on the given LAN Automation session id.
- Invoke this API to get the LAN Automation session logs.
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter. Starting index of the LAN Automation session. Minimum value is 1.
    type: int
  limit:
    description:
    - Limit query parameter. Number of LAN Automation sessions to be retrieved. Limit value can range between 1 to 10.
    type: int
  id:
    description:
    - Id path parameter. LAN Automation session identifier.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for LAN Automation LANAutomationLog
  description: Complete reference of the LANAutomationLog API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-log
- name: Cisco DNA Center documentation for LAN Automation LANAutomationLogById
  description: Complete reference of the LANAutomationLogById API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-log-by-id
notes:
  - SDK Method used are
    lan_automation.LanAutomation.lan_automation_log,
    lan_automation.LanAutomation.lan_automation_log_by_id,

  - Paths used are
    get /dna/intent/api/v1/lan-automation/log,
    get /dna/intent/api/v1/lan-automation/log/{id},

"""

EXAMPLES = r"""
- name: Get all Lan Automation Log
  cisco.dnac.lan_automation_log_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
  register: result

- name: Get Lan Automation Log by id
  cisco.dnac.lan_automation_log_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "nwOrchId": "string",
          "entry": [
            {
              "logLevel": "string",
              "timeStamp": "string",
              "record": "string",
              "deviceId": "string"
            }
          ]
        }
      ],
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright (c) 2022, Julien Riou <julien@riou.xyz>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_bytes
from ansible.module_utils.common.collections import is_string

try:
    from zlib import crc32
    HAS_ZLIB = True
except ImportError:
    HAS_ZLIB = False


DOCUMENTATION = '''
  name: crc32
  short_description: Generate a CRC32 checksum
  version_added: 5.4.0
  description:
    - Checksum a string using CRC32 algorithm and return its hexadecimal representation.
  options:
    _input:
      description:
        - The string to checksum.
      type: string
      required: true
  author:
    - Julien Riou
'''

EXAMPLES = '''
  - name: Checksum a test string
    ansible.builtin.debug:
      msg: "{{ 'test' | community.general.crc32 }}"
'''

RETURN = '''
  _value:
    description: CRC32 checksum.
    type: string
'''


def crc32s(value):
    if not is_string(value):
        raise AnsibleFilterError('Invalid value type (%s) for crc32 (%r)' %
                                 (type(value), value))

    if not HAS_ZLIB:
        raise AnsibleFilterError('Failed to import zlib module')

    data = to_bytes(value, errors='surrogate_or_strict')
    return "{0:x}".format(crc32(data) & 0xffffffff)


class FilterModule:
    def filters(self):
        return {
            'crc32': crc32s,
        }
